﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using GTDemo.GTDemoService;
using AppSrv = Autodesk.AutoCAD.ApplicationServices;
using Autodesk.AutoCAD.DatabaseServices;
using Autodesk.AutoCAD.EditorInput;
using Autodesk.AutoCAD.Geometry;
using Autodesk.AutoCAD.Runtime;
using System.Runtime.InteropServices;
using System.Threading;


namespace GTDemo
{
    // This class contains the code that interacts with the cloud application
    public partial class CloudWindow : UserControl
    {
        
#region "P/Invoke Methods"
        // The following DllImport (called P/Invoke declarations) declarations allow .NET applications to use
        // unmanaged functions (in our case ARX functions).
        [DllImport("acad.exe", CharSet = CharSet.Unicode, CallingConvention = CallingConvention.Cdecl, EntryPoint = "acedPrompt")]
        private static extern int acedPrompt(string handle);
        [DllImport("acad.exe")]
        private static extern void acedPostCommandPrompt();

        [DllImport("acad.exe", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl, EntryPoint = "adsw_acadDocWnd")]
        private static extern System.IntPtr adsw_acadDocWnd();
        [DllImport("user32.dll")]
        private static extern System.IntPtr SetFocus(System.IntPtr hwnd);
#endregion

        // BackgroundWorker is a class provided by .NET 2.0 that makes it easy to perform asynchronous
        // operations in a background thread
        BackgroundWorker _bw;

        public CloudWindow()
        {
            InitializeComponent();
        }
        

        #region "YodaQuote button click implementation"
        // Clicking this button launches a background thread that will keep polling the web
        // service at regular intervals for a random quote of Yoda
        private void yodaQuoteButton_Click(object sender, EventArgs e)
        {
            // We are going to use asynchronous method to call the service. This asynchronocity is on the client side,
            // i.e. this AutoCAD application makes the process asynchronous. For this we use built in .NET class called 
            // BackgroundWorker. This class makes it easy to launch a thread, do our stuff and synchronize back with 
            // the main UI thread, in this case the modeless custom Palette. The other method to asynchronocity is to call
            // the asynchronous web service method. we dont do that here. Please read up on .NET web services  for more information
            if (null == _bw)
            {
                // Instantiate the backgroundworker class
                _bw = new BackgroundWorker
                {
                    WorkerReportsProgress = true,
                    WorkerSupportsCancellation = true
                };
                _bw.DoWork += new DoWorkEventHandler(_bw_DoWork); // This is the event handler that actually makes the call to the web service
                _bw.ProgressChanged += new ProgressChangedEventHandler(_bw_ProgressChanged); // This event handler gets called when there is a change in state
                _bw.RunWorkerCompleted += new RunWorkerCompletedEventHandler(_bw_RunWorkerCompleted); // This event handler gets called when the BackgroundWorker is done.

                _bw.RunWorkerAsync(); // Start the background worker
            }
        }
        #endregion

#region "The BackgroundWorker DoWork event handler implementation"
        // This event handler is where the action is. We call the the Service Method here
        void _bw_DoWork(object sender, DoWorkEventArgs e)
        {
            int counter = 0;
            //Instantiate the .NET Web service on the cloud.
            Service1 srv = new Service1();
            
            // Call the web service method every 5 seconds and report back to the main thread
            // with a new Yoda quote
            while (true)
            {
                String str = srv.YodaQuotes(); // Get the Yoda Quote synchronously
                _bw.ReportProgress(10, str); // Inform the Main thread that there is a new Yoda quote
                Thread.Sleep(5000); // Do this every 5 seconds
                counter += 10; // This counter is just a place holder that may be useful if you have a progress bar
                               // but we dont really need this for your purpose.
            }

        }
#endregion

        #region "The BackgroundWorker ProgressChanged event handler implementation"
        // This event handler gets called when the BackgroundWorker.ReportProgress method
        // gets called in the DoWork event. This event is special because it is run in the
        // Main thread. That is the magic of the BackgroundWorker class
        void _bw_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            yodaQuoteLabel.Text = (string)e.UserState; // Update the Label control with the new Yoda Quote
        }
        #endregion

        #region "The BackgroundWorker RunWorkerCompleted event handler implementation"
        // This event handler gets called when the BackgroundWorker is done with what it is doing.
        // In our implementation we don't do much but just access the exception if any when the background 
        // worker was running.
        void _bw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            System.Exception exp = e.Error; // Get the error if any
        }
        #endregion

        private void label1_Click(object sender, EventArgs e)
        {

        }

        #region "Centroid Button click implementation"
        // In this button event handler, we get the extents of a selected entity and pass it on to the webservice.
        // The web service calculates the centroid based on the extents and sends back the centroid point.
        // We just call the web service method synchronously. No fancy asynchronous calls.
        private void centroidButton_Click(object sender, EventArgs e)
        {
            // Some AutoCAD objects we need to do the job
            AppSrv.Document curDoc = AppSrv.Application.DocumentManager.MdiActiveDocument;
            Database db = curDoc.Database;
            Editor ed = curDoc.Editor;

            // We are accessing the drawing from a modeless dialog, so lock the document
            using (AppSrv.DocumentLock docLoc = curDoc.LockDocument())
            {
                // Set the focus on the AutoCAD editor
                System.IntPtr hwnd = adsw_acadDocWnd();
                SetFocus(hwnd);

                using (Transaction trans = curDoc.TransactionManager.StartTransaction()) // Start transaction
                {
                    // Prompt to select an entity
                    PromptEntityOptions prEntOpts = new PromptEntityOptions("\nSelect an Entity");
                    PromptEntityResult prEntRes = ed.GetEntity(prEntOpts);

                    using (Entity ent = (Entity)trans.GetObject(prEntRes.ObjectId, OpenMode.ForRead))
                    {
                        // Get the extents of the selected entity
                        Extents3d exts = ent.GeometricExtents;

                        // Instantiate a data structure that the cloud service will understand
                        // with the extents values
                        CloudBoundingBox cldBndgBox = new CloudBoundingBox();
                        CloudPoint[] pts = new CloudPoint[2];
                        pts[0] = new CloudPoint();
                        pts[1] = new CloudPoint();
                        cldBndgBox.pts = pts;
                        cldBndgBox.pts[0].x = exts.MinPoint.X;
                        cldBndgBox.pts[0].y = exts.MinPoint.Y;
                        cldBndgBox.pts[0].z = exts.MinPoint.Z;

                        cldBndgBox.pts[1].x = exts.MaxPoint.X;
                        cldBndgBox.pts[1].y = exts.MaxPoint.Y;
                        cldBndgBox.pts[1].z = exts.MaxPoint.Z;

                        // Instantiate the Web Service Object
                        Service1 srv = new Service1();
                        // make the Webservice call synchronously
                        CloudPoint centroid = srv.Centroid(cldBndgBox);

                        StringBuilder strBldr = new StringBuilder();

                        strBldr.Append("X: ");
                        strBldr.Append(centroid.x.ToString());
                        strBldr.Append("\nY: ");
                        strBldr.Append(centroid.y.ToString());
                        strBldr.Append("\nZ: ");
                        strBldr.Append(centroid.z.ToString());

                        // Update the Label with the centroid value
                        centroidLabel.Text = strBldr.ToString();

                        trans.Commit();

                        acedPostCommandPrompt(); // This P/Invoked function allows one to refresh the command line 
                        // if one wants it updated during a modal operation

                    }

                }

            }
        }
        #endregion
        

        private void CloudWindow_Load(object sender, EventArgs e)
        {

        }
    }
}
