﻿////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2010 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. 
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

using System.ServiceModel.Channels;
using System.ServiceModel;

using Inventor;

namespace InventorCloudDemoAddin
{
    public partial class CloudControl : UserControl
    {
        //Inventor application
        private Inventor.Application m_inventorApplication;

        //MFG Database WebService
        private MfgInfoServiceClient mfgSvrClnt;

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // The syncCtrl is used in order to be able to access controls from an event handler
        // Refer to the following devnote for more details:
        // 
        // http://adn.autodesk.com/adn/servlet/devnote?siteID=4814862&id=15443817&linkID=4900509
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private Control syncCtrl;

        private delegate void StatusMessageDelegate(string msg);

        private delegate void CostAndStatusMessageDelegate(double cost, string msg);

        private void DisplayStatusMessage(string msg)
        {
            lbInfoAsync.Text = msg;
        }

        private void DisplayCostAndStatusMessage(double cost, string msg)
        {
            costBox.Text = cost.ToString();

            DisplayStatusMessage(msg);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //    
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        public CloudControl()
        {
            InitializeComponent();

            lbInfoAsync.Text = string.Empty;

            // the control created to help with marshaling needs to be created on the main thread
            syncCtrl = new Control();
            syncCtrl.CreateControl();

            try
            {
                //connect to the web service
                System.ServiceModel.Channels.Binding binding = new BasicHttpBinding();

                EndpointAddress endpointAddressMfg = new EndpointAddress("http://adnmonitor.cloudapp.net/MfgInfoService.svc?wsdl");
                mfgSvrClnt = new MfgInfoServiceClient(binding, endpointAddressMfg);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Unable to connect to the web service: " +
                    System.Environment.NewLine + "http://adnmonitor.cloudapp.net/MfgInfoService.svc?wsdl" +
                    System.Environment.NewLine + "Check your Internet connection...",
                    "Web Service connection failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        //    
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        public void Initialize(Inventor.Application InvApplication)
        {
            m_inventorApplication = InvApplication;

            //fills up the materail combobox with material names from a part

            AssemblyDocument asm = m_inventorApplication.ActiveDocument as AssemblyDocument;

            foreach (ComponentOccurrence occurrence in asm.ComponentDefinition.Occurrences.get_AllLeafOccurrences(null))
            {
                if (occurrence.DefinitionDocumentType != DocumentTypeEnum.kPartDocumentObject)
                    continue;

                PartDocument part = occurrence.Definition.Document as PartDocument;

                PartComponentDefinition partCompDef = part.ComponentDefinition as PartComponentDefinition;

                cbMaterial.Items.Clear();

                foreach (Material material in part.Materials)
                {
                    cbMaterial.Items.Add(material.Name);
                }

                cbMaterial.SelectedIndex = 0;

                //Done after first part
                return;
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Upload material info to the cloud (create or update)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void uploadButton_Click(object sender, EventArgs e)
        {
            if (matSpecBox.Text == string.Empty ||
                matUnitBox.Text == string.Empty ||
                regionBox.Text == string.Empty ||
                costBox.Text == string.Empty)
            {
                return;
            }

            try
            {
                float cost = float.Parse(costBox.Text);

                string matName = cbMaterial.SelectedItem.ToString();

                //Upload synchron
                /*mfgSvrClnt.UpdateMfgMaterialInfo(matName, 
                    matSpecBox.Text, 
                    matUnitBox.Text, 
                    regionBox.Text, 
                    cost);*/

                //upload asynchron 
                mfgSvrClnt.BeginUpdateMfgMaterialInfo(matName,
                    matSpecBox.Text,
                    matUnitBox.Text,
                    regionBox.Text,
                    cost,
                    UpdateMfgMaterialInfoResponder,
                    matName);
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message,
                    "Update Information Failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        //Asychron callback
        private void UpdateMfgMaterialInfoResponder(IAsyncResult result)
        {
            try
            {
                mfgSvrClnt.EndUpdateMfgMaterialInfo(result);

                if (result.IsCompleted)
                {
                    string msg = "Material \"" + result.AsyncState.ToString() + "\""
                        + System.Environment.NewLine + "Successfully uploaded!";

                    syncCtrl.Invoke(new StatusMessageDelegate(DisplayStatusMessage), new object[] { msg });
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message,
                    "Update Information Failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Get material cost info from the cloud
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void getButton_Click(object sender, EventArgs e)
        {
            if (matSpecBox.Text == string.Empty ||
                matUnitBox.Text == string.Empty ||
                regionBox.Text == string.Empty)
            {
                return;
            }

            try
            {
                string matName = cbMaterial.SelectedItem.ToString();

                // Get the cost per unit from MFG Database - synchron
                /*double cost = mfgSvrClnt.GetCostPerUnit(matName, 
                    matSpecBox.Text, 
                    matUnitBox.Text, 
                    regionBox.Text);
                 
                costBox.Text = cost.ToString();*/

                //Get cost asynchron
                mfgSvrClnt.BeginGetCostPerUnit(matName,
                    matSpecBox.Text,
                    matUnitBox.Text,
                    regionBox.Text,
                    GetCostPerUnitInfoResponder,
                    matName);
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message,
                    "Get Information Failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        //Asychron callback
        private void GetCostPerUnitInfoResponder(IAsyncResult result)
        {
            try
            {
                double cost = mfgSvrClnt.EndGetCostPerUnit(result);

                if (result.IsCompleted)
                {
                    string msg = "Material \"" + result.AsyncState.ToString() + "\""
                        + System.Environment.NewLine + "Information retrieved!";

                    syncCtrl.Invoke(new CostAndStatusMessageDelegate(DisplayCostAndStatusMessage), new object[] { cost, msg });
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message,
                    "Get Information Failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Update BOM properties from the cloud
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void bUpdateBOM_Click(object sender, EventArgs e)
        {
            AssemblyDocument asm = m_inventorApplication.ActiveDocument as AssemblyDocument;

            foreach (ComponentOccurrence occurrence in asm.ComponentDefinition.Occurrences.get_AllLeafOccurrences(null))
            {
                if (occurrence.DefinitionDocumentType != DocumentTypeEnum.kPartDocumentObject)
                    continue;

                PartDocument part = occurrence.Definition.Document as PartDocument;

                PartComponentDefinition partCompDef = part.ComponentDefinition as PartComponentDefinition;

                Material material = partCompDef.Material;

                try
                {
                    mfgSvrClnt.BeginGetCostPerUnit(material.Name,
                       matSpecBox.Text,
                       matUnitBox.Text,
                       regionBox.Text,
                       UpdateBOMInfoResponder,
                       occurrence.Name);
                }
                catch(Exception ex)
                {
                    System.Windows.Forms.MessageBox.Show(ex.Message,
                        "Update BOM Failed Failed!",
                        MessageBoxButtons.OK,
                        MessageBoxIcon.Error);
                }
            }
        }

        //Asychron callback
        private void UpdateBOMInfoResponder(IAsyncResult result)
        {
            try
            {
                double cost = mfgSvrClnt.EndGetCostPerUnit(result);

                if (result.IsCompleted)
                {
                    AssemblyDocument asm = m_inventorApplication.ActiveDocument as AssemblyDocument;

                    ComponentOccurrence occurrence = asm.ComponentDefinition.Occurrences.get_ItemByName(result.AsyncState.ToString());

                    PartDocument part = occurrence.Definition.Document as PartDocument;

                    PartComponentDefinition partCompDef = part.ComponentDefinition as PartComponentDefinition;

                    PropertySet customSet = part.PropertySets["{D5CDD505-2E9C-101B-9397-08002B2CF9AE}"];

                    Inventor.Property volumeProp = customSet["Volume"];

                    volumeProp.Value = partCompDef.MassProperties.Volume;

                    cost = cost * partCompDef.MassProperties.Volume * 0.001;

                    Inventor.Property costProp = customSet["Cost"];

                    costProp.Value = cost;

                    string msg = "Occurrence \"" + result.AsyncState.ToString() + "\""
                       + System.Environment.NewLine + "Updated!";

                    syncCtrl.Invoke(new StatusMessageDelegate(DisplayStatusMessage), new object[] { msg });
                }
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message,
                    "Update BOM Failed Failed!",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Not exposed through the control - I used this method to upload material info to the cloud (with random costs)
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void bUploadAll_Click(object sender, EventArgs e)
        {
            try
            {
                AssemblyDocument asm = m_inventorApplication.ActiveDocument as AssemblyDocument;

                foreach (ComponentOccurrence occurrence in asm.ComponentDefinition.Occurrences.get_AllLeafOccurrences(null))
                {
                    if (occurrence.DefinitionDocumentType != DocumentTypeEnum.kPartDocumentObject)
                        continue;

                    PartDocument part = occurrence.Definition.Document as PartDocument;

                    PartComponentDefinition partCompDef = part.ComponentDefinition as PartComponentDefinition;

                    foreach (Material material in part.Materials)
                    {
                        int cost = RandomNumber(10, 1000);

                        mfgSvrClnt.UpdateMfgMaterialInfo(material.Name,
                            "DevDays Material Demo",
                            "M^3",
                            "ADN WORLDWIDE",
                            cost);
                    }
                }
            }
            catch
            { 
                
            }
        }

        private int RandomNumber(int min, int max)
        {
            Random random = new Random();
            return random.Next(min, max);
        }
    }
}
