////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Autodesk, Inc. All rights reserved 
// Written by Philippe Leefsma 2010 - ADN/Developer Technical Services
//
// This software is provided as is, without any warranty that it will work. You choose to use this tool at your own risk.
// Neither Autodesk nor the author Philippe Leefsma can be taken as responsible for any damage this tool can cause to 
// your data. 
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Runtime.InteropServices;
using Inventor;
using Microsoft.Win32;

namespace InventorCloudDemoAddin
{
    /// <summary>
    /// This is the primary AddIn Server class that implements the ApplicationAddInServer interface
    /// that all Inventor AddIns are required to implement. The communication between Inventor and
    /// the AddIn is via the methods on this interface.
    /// </summary>
    [GuidAttribute("0b1bde94-0367-4a18-8a56-c251bb7100f2")]
    public class StandardAddInServer : Inventor.ApplicationAddInServer
    {
        // Inventor application object.
        private Inventor.Application m_inventorApplication;

        //The cloud control button definition
        private ButtonDefinition _CloudControlButtonDef;

        public StandardAddInServer()
        {

        }

        #region ApplicationAddInServer Members

        public void Activate(Inventor.ApplicationAddInSite addInSiteObject, bool firstTime)
        {
            // This method is called by Inventor when it loads the addin.
            // The AddInSiteObject provides access to the Inventor Application object.
            // The FirstTime flag indicates if the addin is loaded for the first time.

            // Initialize AddIn members.
            m_inventorApplication = addInSiteObject.Application;

            // TODO: Add ApplicationAddInServer.Activate implementation.
            // e.g. event initialization, command creation etc.
            CreateControls(firstTime);
        }

        public void Deactivate()
        {
            // This method is called by Inventor when the AddIn is unloaded.
            // The AddIn will be unloaded either manually by the user or
            // when the Inventor session is terminated

            // TODO: Add ApplicationAddInServer.Deactivate implementation

            // Release objects.
            Marshal.ReleaseComObject(m_inventorApplication);
            m_inventorApplication = null;

            GC.WaitForPendingFinalizers();
            GC.Collect();
        }

        public void ExecuteCommand(int commandID)
        {
            // Note:this method is now obsolete, you should use the 
            // ControlDefinition functionality for implementing commands.
        }

        public object Automation
        {
            // This property is provided to allow the AddIn to expose an API 
            // of its own to other programs. Typically, this  would be done by
            // implementing the AddIn's API interface in a class and returning 
            // that class object through this property.

            get
            {
                // TODO: Add ApplicationAddInServer.Automation getter implementation
                return null;
            }
        }

        #endregion

        #region COM Registration functions

        /// <summary>
        /// Registers this class as an Add-In for Autodesk Inventor.
        /// This function is called when the assembly is registered for COM.
        /// </summary>
        [ComRegisterFunctionAttribute()]
        public static void Register(Type t)
        {
            RegistryKey clssRoot = Registry.ClassesRoot;
            RegistryKey clsid = null;
            RegistryKey subKey = null;

            try
            {
                clsid = clssRoot.CreateSubKey("CLSID\\" + AddInGuid(t));
                clsid.SetValue(null, "InventorCloudDemoAddin");
                subKey = clsid.CreateSubKey("Implemented Categories\\{39AD2B5C-7A29-11D6-8E0A-0010B541CAA8}");
                subKey.Close();

                subKey = clsid.CreateSubKey("Settings");
                subKey.SetValue("AddInType", "Standard");
                subKey.SetValue("LoadOnStartUp", "1");

                //subKey.SetValue("SupportedSoftwareVersionLessThan", "");
                subKey.SetValue("SupportedSoftwareVersionGreaterThan", "12..");
                //subKey.SetValue("SupportedSoftwareVersionEqualTo", "");
                //subKey.SetValue("SupportedSoftwareVersionNotEqualTo", "");
                //subKey.SetValue("Hidden", "0");
                //subKey.SetValue("UserUnloadable", "1");
                subKey.SetValue("Version", 2);
                subKey.Close();

                subKey = clsid.CreateSubKey("Description");
                subKey.SetValue(null, "Cloud Demo Inventor add-in");
            }
            catch
            {
                System.Diagnostics.Trace.Assert(false);
            }
            finally
            {
                if (subKey != null) subKey.Close();
                if (clsid != null) clsid.Close();
                if (clssRoot != null) clssRoot.Close();
            }

        }

        /// <summary>
        /// Unregisters this class as an Add-In for Autodesk Inventor.
        /// This function is called when the assembly is unregistered.
        /// </summary>
        [ComUnregisterFunctionAttribute()]
        public static void Unregister(Type t)
        {
            RegistryKey clssRoot = Registry.ClassesRoot;
            RegistryKey clsid = null;

            try
            {
                clssRoot = Microsoft.Win32.Registry.ClassesRoot;
                clsid = clssRoot.OpenSubKey("CLSID\\" + AddInGuid(t), true);
                clsid.SetValue(null, "");
                clsid.DeleteSubKeyTree("Implemented Categories\\{39AD2B5C-7A29-11D6-8E0A-0010B541CAA8}");
                clsid.DeleteSubKeyTree("Settings");
                clsid.DeleteSubKeyTree("Description");
            }
            catch { }
            finally
            {
                if (clsid != null) clsid.Close();
                if (clssRoot != null) clssRoot.Close();
            }
        }

        // This function uses reflection to get the value for the GuidAttribute attached to the class.
        private static String AddInGuid(Type t)
        {
            string guid = "";

            try
            {
                Object[] customAttributes = t.GetCustomAttributes(typeof(GuidAttribute), false);
                GuidAttribute guidAttribute = (GuidAttribute)customAttributes[0];
                guid = "{" + guidAttribute.Value.ToString() + "}";
            }
            catch
            {
            }

            return guid;

        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Performs UI Initialization Classic/Ribbon, loads resources
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        public void CreateControls(bool firstTime)
        {
            ControlDefinitions ctrlDefs = m_inventorApplication.CommandManager.ControlDefinitions;

            System.Reflection.Assembly oAssembly = System.Reflection.Assembly.GetExecutingAssembly();

            //string[] resources = oAssembly.GetManifestResourceNames();

            System.IO.Stream stream32 = oAssembly.GetManifestResourceStream("InventorCloudDemoAddin.resources.Cloud 32x32.ico");
            System.IO.Stream stream16 = oAssembly.GetManifestResourceStream("InventorCloudDemoAddin.resources.Cloud 32x32.ico");

            System.Drawing.Icon Icon32 = new System.Drawing.Icon(stream32);
            System.Drawing.Icon Icon16 = new System.Drawing.Icon(stream16);

            object IPictureDisp32 = Microsoft.VisualBasic.Compatibility.VB6.Support.IconToIPicture(Icon32);
            object IPictureDisp16 = Microsoft.VisualBasic.Compatibility.VB6.Support.IconToIPicture(Icon16);

            try
            {
                _CloudControlButtonDef = ctrlDefs["Autodesk:CloudDemoAddin:CloudCtrl"] as ButtonDefinition;
            }
            catch
            {
                _CloudControlButtonDef = ctrlDefs.AddButtonDefinition("Cloud Control",
                                                          "Autodesk:CloudDemoAddin:CloudCtrl",
                                                          CommandTypesEnum.kEditMaskCmdType,
                                                          AddInGuid(typeof(StandardAddInServer)),
                                                          "Cloud Control",
                                                          "Displays Cloud Control",
                                                          IPictureDisp16,
                                                          IPictureDisp32,
                                                          ButtonDisplayEnum.kDisplayTextInLearningMode);
            }

            _CloudControlButtonDef.OnExecute += new ButtonDefinitionSink_OnExecuteEventHandler(CloudControlButtonDef_OnExecute);
           
            if (firstTime)
            {
                if (IsRibbonUI())
                {
                    DoInitRibbonUI();
                }
                else
                {
                    //Not handled here...
                    //DoInitClassicUI();
                }
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Determine UI Style Classic/Ribbon
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        bool IsRibbonUI()
        {
            //Ribbon start from R14 (Inventor 2010)
            if (m_inventorApplication.SoftwareVersion.Major < 14)
                return false;

            //UI can be set to classic, so need to double check
            if (m_inventorApplication.UserInterfaceManager.InterfaceStyle != InterfaceStyleEnum.kRibbonInterface)
                return false;

            return true;
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Ribbon UI Intitialization
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        void DoInitRibbonUI()
        {
            Ribbon assemblyRibbon = m_inventorApplication.UserInterfaceManager.Ribbons["Assembly"];
            RibbonTab assemblyTab = assemblyRibbon.RibbonTabs["id_TabTools"];

            RibbonPanel asmPanel = assemblyTab.RibbonPanels.Add("Developer Days 2010",
                "Autodesk:CloudDemoAddin:AsmPanel",
                AddInGuid(typeof(StandardAddInServer)),
                "", false);

            asmPanel.CommandControls.AddButton(_CloudControlButtonDef, true, true, "", false);
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Main Control Execute event: displays control in Inventor Pane and 
        //    
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        void CloudControlButtonDef_OnExecute(NameValueMap Context)
        {
            createBrowserPane();
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Use: Create cloud control in a dockable window - not used
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void createDockableWindow()
        {
            try
            {
                DockableWindow window = m_inventorApplication.UserInterfaceManager.DockableWindows.Add("0b1bde94-0367-4a18-8a56-c251bb7100f2",
                    "Autodesk.adn.clouddemo",
                    "Cloud Demo");

                window.AddChild("InventorCloudDemoAddin.CloudControl");

                InventorCloudDemoAddin.CloudControl control = window.Control as InventorCloudDemoAddin.CloudControl;

                control.Initialize(m_inventorApplication);

                window.DisabledDockingStates = DockingStateEnum.kDockBottom | DockingStateEnum.kDockTop;
                window.DockingState = DockingStateEnum.kDockLeft;

                window.Visible = true;
            }
            catch
            { 
                
            }
        }

        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        // Use: Create cloud control in browser pane
        //
        ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
        private void createBrowserPane()
        {
            try
            {
                Document doc = m_inventorApplication.ActiveDocument;

                try
                {
                    BrowserPane browserPane = doc.BrowserPanes["Cloud Demo"];
                    browserPane.Activate();
                }
                catch
                {
                    BrowserPane browserPane = doc.BrowserPanes.Add("Cloud Demo", "InventorCloudDemoAddin.CloudControl");

                    InventorCloudDemoAddin.CloudControl control = browserPane.Control as InventorCloudDemoAddin.CloudControl;

                    control.Initialize(m_inventorApplication);

                    browserPane.Activate();
                }
            }
            catch
            { 
                
            }
        }
    }
}
