﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Autodesk.AutoCAD.DatabaseServices;
using AcadApp = Autodesk.AutoCAD.ApplicationServices.Application;
using Autodesk.Civil.ApplicationServices;
using Autodesk.Civil.PipeNetwork.DatabaseServices;
using System.ServiceModel;

namespace PipeNetworkCost
{
    public partial class PalettePipeNetworkCost : UserControl
    {
        AecInfoServiceClient aecSvrClnt; // For AEC Database

        #region internal structure to show information on the Network combo box
        public struct NameAndObjectId
        {
            public string name;
            public ObjectId objectId;

            public NameAndObjectId(string n, ObjectId id)
            {
                name = n;
                objectId = id;
            }

            public override string ToString() { return name; }
        }
        #endregion

        #region user control events
        public PalettePipeNetworkCost()
        {
            InitializeComponent();

            System.ServiceModel.Channels.Binding binding = new BasicHttpBinding();

            //// For AEC Database
            EndpointAddress endpointAddressAec = new EndpointAddress("http://adnmonitor.cloudapp.net/AecInfoService.svc?wsdl");
            aecSvrClnt = new AecInfoServiceClient(binding, endpointAddressAec);
        }

        private void btnRefreshPipeNetworkList_Click(object sender, EventArgs e)
        {
            RefreshPipeNetworkList();
        }

        private void PalettePipeNetworkCost_Load(object sender, EventArgs e)
        {
            RefreshPipeNetworkList();
        }

        private void cboPipeNetworkList_SelectedIndexChanged(object sender, EventArgs e)
        {
            RefreshPartList();
            RefreshMaterialList();
        }
        #endregion

        #region filling methods
        private void RefreshPipeNetworkList()
        {
            Database db = AcadApp.DocumentManager.MdiActiveDocument.Database;
            using (Transaction trans = db.TransactionManager.StartTransaction())
            {
                CivilDocument civilDoc = CivilApplication.ActiveDocument;
                ObjectIdCollection pipeNetworkIds = civilDoc.GetPipeNetworkIds();
                cboPipeNetworkList.Items.Clear();
                foreach (ObjectId networkId in pipeNetworkIds)
                {
                    Network network = trans.GetObject(networkId, OpenMode.ForRead) as Network;
                    cboPipeNetworkList.Items.Add(new NameAndObjectId(network.Name, networkId));
                }
                trans.Commit();
            }
        }

        private void RefreshMaterialList()
        {
            Database db = AcadApp.DocumentManager.MdiActiveDocument.Database;
            using (Transaction trans = db.TransactionManager.StartTransaction())
            {
                lstMaterials.Items.Clear();
                foreach (KeyValuePair<string, double> key in _materialIdsAndVolume)
                {
                    double materialCost = (GetMaterialPrice(key.Key) * key.Value);

                    ListViewItem materialItem = new ListViewItem(
                        new string[] { key.Key, key.Value.ToString("0.00"), materialCost.ToString("0.00") });
                    lstMaterials.Items.Add(materialItem);
                }
                lstMaterials.Columns[0].Width = -1;
                trans.Commit();
            }
        }

        private void RefreshPartList()
        {
            if (cboPipeNetworkList.SelectedIndex < 0) return;
            NameAndObjectId selectedPipeNetwork = (NameAndObjectId)cboPipeNetworkList.SelectedItem;
            lstPipeNetworkParts.Items.Clear();

            _materialIdsAndVolume = new Dictionary<string, double>();

            Database db = AcadApp.DocumentManager.MdiActiveDocument.Database;
            using (Transaction trans = db.TransactionManager.StartTransaction())
            {
                Network network = trans.GetObject(selectedPipeNetwork.objectId, OpenMode.ForRead) as Network;

                AddPartsToList(trans, network.GetPipeIds());//*
                AddPartsToList(trans, network.GetStructureIds());//*

                trans.Commit();
            }
        }

        Dictionary<string, double> _materialIdsAndVolume;

        private void AddPartsToList(Transaction trans, ObjectIdCollection partIds)
        {
            foreach (ObjectId pipeId in partIds)
            {
                Part part = trans.GetObject(pipeId, OpenMode.ForRead) as Part;

                double partVolume = part.Solid3dBody.MassProperties.Volume;
                double partCost = (GetMaterialPrice(part.Material) * partVolume);

                ListViewItem partItem = new ListViewItem(
                    new string[] { part.Name, part.Material, partVolume.ToString("0.00"), partCost.ToString("0.00") });
                lstPipeNetworkParts.Items.Add(partItem);

                if (_materialIdsAndVolume.ContainsKey(part.Material))
                    _materialIdsAndVolume[part.Material] += partVolume;
                else
                    _materialIdsAndVolume.Add(part.Material, partVolume);
            }

            lstPipeNetworkParts.Columns[0].Width = -1;
            lstPipeNetworkParts.Columns[1].Width = -1;
        }
        #endregion

        #region Material cost methods
        //improve storing values locally (cache)
        //Dictionary<string, double> _materialCost;

        private double GetMaterialPrice(String materialName)
        {
            double cost = 0.0;
            try
            {
                cost = aecSvrClnt.GetCostPerUnit(materialName,
                    materialName,
                    "M^3",
                    "World");
            }
            catch (Exception ex)
            {

            }
            return cost;
        }

        #endregion
    }
}
